package com.alibaba.tesseract.page.activity;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;


import com.alibaba.android.tesseract.core.event.base.ISubscriber;
import com.alibaba.android.tesseract.core.instance.ITesseract;
import com.alibaba.android.tesseract.core.instance.ITesseractInstance;
import com.alibaba.android.tesseract.core.instance.TesseractInstanceConfig;
import com.alibaba.android.tesseract.core.request.IRequestCallback;
import com.alibaba.android.tesseract.core.request.Request;
import com.alibaba.android.tesseract.core.request.RequestBuilder;
import com.alibaba.fastjson.JSON;
import com.alibaba.gov.android.api.account.IUserManagerService;
import com.alibaba.gov.android.api.account.UserInfo;
import com.alibaba.gov.android.api.login.ILoginService;
import com.alibaba.gov.android.api.network.request.body.ZWJSONObjectRequestBody;
import com.alibaba.gov.android.api.site.ISiteMidService;
import com.alibaba.gov.android.api.site.SiteModel;
import com.alibaba.gov.android.api.zwmonitor.IAppMonitorApi;
import com.alibaba.gov.android.api.zwmonitor.IUserOperationApi;
import com.alibaba.gov.android.api.zwmonitor.IZWMonitor;
import com.alibaba.gov.android.common.config.AppConfig;
import com.alibaba.gov.android.foundation.utils.ToastUtil;
import com.alibaba.gov.android.navigation.UniversalNavigationBar;
import com.alibaba.tesseract.page.PageCache;
import com.alibaba.tesseract.page.events.RefreshPageEvent;
import com.alibaba.tesseract.page.utils.BaseSubscribeUtil;
import com.alibaba.tesseract.page.events.ModifyDataEvent;
import com.alibaba.android.tesseract.sdk.common.model.IDMComponent;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.gov.android.foundation.base.fragment.BaseFragment;
import com.alibaba.gov.android.servicebus.manager.ServiceManager;
import com.alibaba.gov.android.tesseract.page.R;
import com.alibaba.tesseract.page.inter.ILifecycleCallback;
import com.alibaba.tesseract.page.inter.ILoadMoreListener;
import com.alibaba.tesseract.page.inter.IRefreshListener;
import com.alibaba.tesseract.page.utils.RVHeightUtils;
import com.scwang.smartrefresh.layout.SmartRefreshLayout;
import com.scwang.smartrefresh.layout.api.RefreshLayout;
import com.scwang.smartrefresh.layout.listener.OnLoadMoreListener;
import com.scwang.smartrefresh.layout.listener.OnRefreshListener;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import static com.alibaba.tesseract.page.activity.Constants.MULTI_PAGE_REQUEST_API;
import static com.alibaba.tesseract.page.activity.Constants.REQUEST_TYPE_LOAD;
import static com.alibaba.tesseract.page.activity.Constants.REQUEST_TYPE_LOGIN;
import static com.alibaba.tesseract.page.activity.Constants.REQUEST_TYPE_NORMAL_REFRESH;
import static com.alibaba.tesseract.page.activity.Constants.REQUEST_TYPE_PULL_FRESH;
import static com.alibaba.tesseract.page.activity.Constants.REQUEST_TYPE_RESUME;
import static com.alibaba.tesseract.page.activity.Constants.REQUEST_TYPE_SITE_CHANGE;

/**
 * Created by yiwei on 2020-04-08.
 */
public class BaseTesseractFragment extends BaseFragment implements OnRefreshListener, OnLoadMoreListener {
    public ITesseractInstance mTesseractInstance;
    protected SmartRefreshLayout mRefreshLayout;
    protected FrameLayout mBody;
    protected String mSinglePageRequestUrl;
    protected String mMultiPageRequestUrl;
    private String mSiteId;
    private String mPageId;
    private String mUrlTitle;
    private String mPageName;
    protected boolean mEnableLoadMore = true;
    protected boolean mEnablePullRefresh = true;
    //同时禁止recyclerview滚动、下拉刷新、加载更多
    protected boolean mDisableAllScroll = false;
    //禁止recyclerview滚动
    protected boolean mEnableScrollVertical = true;
    protected boolean mEnableLoginReceive = false;
    protected boolean mEnableResumeRefresh = false;
    protected boolean mMgopRequest = false;
    protected boolean mHideHeaderBar = false;
    protected boolean mHideLoading = false;
    protected boolean mMgopRequestNeedLogin = false;
    private Map<String, String> mRequestHeader;
    private FrameLayout mHeaderLayout;
    private FrameLayout mBottomLayout;
    private View mHeaderView;
    private UniversalNavigationBar mNavBar;
    private View mBottomView;

    private HashMap<String, String> mMonitorInfo;
    private IRefreshListener refreshListener;
    private Map<String, ISubscriber> mEventSubscriber;
    private ILoadMoreListener loadMoreListener;
    private ILoginService mLoginService;
    private LoginStateChangeReceiver mLoginBroadcast;
    private ILifecycleCallback mLifecycle;
    private PageCache mPageCache;
    private JSONObject mLastGlobal;
    private String mPageBgColor = "#ffffff";

    @Override
    public int getLayoutResId() {
        return R.layout.base_tesseract_fragment_layout;
    }

    @Override
    public void findViews(View rootView) {
        mRefreshLayout = rootView.findViewById(R.id.tesseract_fragment_refreshlayout);
        mHeaderLayout = rootView.findViewById(R.id.tesseract_fragment_header);
        mBottomLayout = rootView.findViewById(R.id.tesseract_fragment_bottom);
        mBody = rootView.findViewById(R.id.tesseract_fragment_body);
    }

    @Override
    public boolean needEventBus() {
        return true;
    }

    @Override
    public void initViews(View view) {
        initMonitorInfo();
        initHeaderView();
        initBottomView();
        initTesseract();
        initRecyclerView();
        initMultiReqUrl();
        loadCacheDataIfNeed();
        registerLoginReceive();
        initRefreshLayout();
    }


    private void initRefreshLayout() {
        toggleRefreshLayoutLoadMore(mEnableLoadMore);
        toggleRefreshLayoutPullRefresh(mEnablePullRefresh);
        mRefreshLayout.setEnableOverScrollDrag(mEnablePullRefresh && mEnableLoadMore);
    }

    private void initMultiReqUrl() {
        mMultiPageRequestUrl = AppConfig.getString("baseHost") + MULTI_PAGE_REQUEST_API;
    }

    private void loadCacheDataIfNeed() {
        if (mPageCache == null || mTesseractInstance == null) {
            return;
        }
        JSONObject cacheData = mPageCache.getCacheData(getContext(), pageName());
        if (cacheData == null || cacheData.isEmpty()) {
            return;
        }
        mTesseractInstance.renderData(cacheData);
    }

    public void registerLifecycle(ILifecycleCallback lifecycle) {
        this.mLifecycle = lifecycle;
    }

    private void registerLoginReceive() {
        mLoginBroadcast = new LoginStateChangeReceiver();
        mLoginService = ServiceManager.getInstance().getService(ILoginService.class.getName());
        if (mLoginService != null) {
            mLoginService.registerLoginReceiver(mLoginBroadcast);
            return;
        }
    }

    protected String pageName() {
        if (!TextUtils.isEmpty(mPageName)) {
            return "Page_Tesseract_" + mPageName;
        } else if (!TextUtils.isEmpty(mPageId)) {
            return "Page_Tesseract_" + mPageId;
        }
        return this.getClass().getSimpleName();
    }

    private void initMonitorInfo() {
        mMonitorInfo = new HashMap<>();
        mMonitorInfo.put("title", mUrlTitle);
        mMonitorInfo.putAll(parseMonitorParamFromUrl(mSinglePageRequestUrl));

        IUserOperationApi userMonitor = getUserMonitor();
        if (userMonitor != null) {
            userMonitor.pageIn(pageName(), null);
            userMonitor.updatePageName(pageName());
        }
        IAppMonitorApi appMonitorApi = getAppMonitor();
        if (appMonitorApi != null) {
            appMonitorApi.commitSuccess(pageName(), "Page_Load", mMonitorInfo);
        }
    }


    /**
     * tesseract容器与服务端特定参数
     *
     * @param url
     */
    private void parseParamsFromUrl(String url) {
        if (TextUtils.isEmpty(url)) {
            return;
        }
        Uri uri = Uri.parse(url);
        mEnableLoginReceive = uri.getBooleanQueryParameter(Constants.LOGIN_RECEIVE, false);
        mEnableResumeRefresh = uri.getBooleanQueryParameter(Constants.RESUME_REFRESH, false);
        mEnableScrollVertical = uri.getBooleanQueryParameter(Constants.SCROLL_VERTICAL, true);
        //如果通过方法禁用了，不再走url解析
        if (mEnableLoadMore) {
            mEnableLoadMore = uri.getBooleanQueryParameter(Constants.LOAD_MORE, true);
        }
        //如果通过方法禁用了，不再走url解析
        if (mEnablePullRefresh) {
            mEnablePullRefresh = uri.getBooleanQueryParameter(Constants.PULL_REFRESH, true);
        }
        //如果通过方法禁用了，不再走url解析
        if (!mHideHeaderBar) {
            mHideHeaderBar = uri.getBooleanQueryParameter(Constants.HIDE_HEADER_BAR, false);
        }
        mHideLoading = uri.getBooleanQueryParameter(Constants.HIDE_LOADING, false);
        mMgopRequest = uri.getBooleanQueryParameter(Constants.MGOP_REQUEST, false);
        mPageId = getQueryParamterOrDefault(uri, Constants.PAGE_ID, "");
        mUrlTitle = getQueryParamterOrDefault(uri, Constants.PAGE_TITLE, "");
        mDisableAllScroll = uri.getBooleanQueryParameter(Constants.FORBID_SCROLL, false);
        if (mDisableAllScroll) {
            mEnableScrollVertical = false;
            mEnablePullRefresh = false;
            mEnableLoadMore = false;
        }
    }

    private String getQueryParamterOrDefault(Uri uri, String key, String defaultValue) {
        String result = uri.getQueryParameter(key);
        return result != null ? result : defaultValue;
    }

    protected Map<String, String> parseMonitorParamFromUrl(String url) {
        Map<String, String> map = new HashMap<>();
        if (!TextUtils.isEmpty(url)) {
            Uri uri = Uri.parse(url);
            Set set = uri.getQueryParameterNames();
            if (set == null) {
                return map;
            }
            Iterator<String> iterator = set.iterator();
            while (iterator.hasNext()) {
                String key = iterator.next();
                map.put(key, uri.getQueryParameter(key));
            }
        }
        return map;
    }

    private String parseAndSetSiteId(String url) {
        if (TextUtils.isEmpty(url)) {
            return "";
        }
        Uri uri = Uri.parse(url);
        mSiteId = uri.getQueryParameter("siteId");
        if (TextUtils.isEmpty(mSiteId)) {
            ISiteMidService siteService = ServiceManager.getInstance().getService(ISiteMidService.class.getName());
            if (siteService != null) {
                SiteModel siteModel = siteService.getGlobalSite();
                if (siteModel == null) {
                    siteModel = siteService.getDefaultSite();
                }
                mSiteId = siteModel.code;
            }
        }
        if (url.contains("?")) {
            url += "&siteId=" + mSiteId;
        } else {
            url += "?siteId=" + mSiteId;
        }

        return url;
    }

    private void initHeaderView() {
        if (mHideHeaderBar) {
            mHeaderLayout.setVisibility(View.GONE);
            return;
        }
        if (mHeaderView == null) {
            mHeaderView = initDefaultHeaderView();
        }
        mHeaderLayout.removeAllViews();
        mHeaderLayout.addView(mHeaderView);
        mHeaderLayout.setVisibility(View.VISIBLE);
    }

    private void initBottomView() {
        if (mBottomView == null) {
            mBottomLayout.setVisibility(View.GONE);
            return;
        }
        mBottomLayout.removeAllViews();
        mBottomLayout.addView(mBottomView);
        mBottomLayout.setVisibility(View.VISIBLE);
    }

    private View initDefaultHeaderView() {
        mNavBar = new UniversalNavigationBar(getContext());
        setTitle(mUrlTitle);
        showShareBtn(mUrlTitle);
        return mNavBar.getUniversalNavigationBarView();
    }

    public void setPageBackgroundColor(String bgColor) {
        this.mPageBgColor = bgColor;
    }

    public void setMultiPageRequestUrl(String requestUrl) {
        this.mMultiPageRequestUrl = requestUrl;
    }

    public void setRequestHeader(Map<String, String> requestHeader) {
        this.mRequestHeader = requestHeader;
    }

    public void setHeaderView(View headerView) {
        this.mHeaderView = headerView;
    }

    public void setBottomView(View bottomView) {
        this.mBottomView = bottomView;
    }

    private void initTesseract() {
        ITesseract tesseract = ServiceManager.getInstance().getService(ITesseract.class.getName());
        if (tesseract == null) {
            return;
        }
        mTesseractInstance = tesseract.createInstance(getContext(),
                new TesseractInstanceConfig()
                        .pageName(pageName())
                        .requestNeedLogin(mMgopRequestNeedLogin)
                        .backgroundColor(Color.parseColor(mPageBgColor))
                        .setComponentDebugMark(1002));
        BaseSubscribeUtil.initEventSubsriber(mTesseractInstance, mEventSubscriber);
        mBody.removeAllViews();
        mBody.addView(mTesseractInstance.getRootView());
    }

    private HashMap<String, String> makeHeaders() {
        HashMap<String, String> headers = new HashMap<>();
        if (mRequestHeader != null && !mRequestHeader.isEmpty()) {
            headers.putAll(mRequestHeader);
        }
        IUserManagerService userManagerService = ServiceManager.getInstance().getService(IUserManagerService.class.getName());
        if (userManagerService == null) {
            return headers;
        }
        UserInfo userInfo = userManagerService.getUserInfo();
        if (userInfo == null) {
            return headers;
        }
        String token = userInfo.getToken();
        if (!TextUtils.isEmpty(token)) {
            headers.put("token", token);
        }
        return headers;
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        if (isVisibleToUser && isVisible()) {
            onVisible();
        }
        super.setUserVisibleHint(isVisibleToUser);
    }

    @Override
    public void onResume() {
        super.onResume();
        if (getUserVisibleHint() && !isHidden()) {
            onVisible();
        }
    }

    private void onVisible() {
        if (mEnableResumeRefresh) {
            doRefresh(mSinglePageRequestUrl, REQUEST_TYPE_RESUME);
        }
        if (mLifecycle != null) {
            mLifecycle.onResume();
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (EventBus.getDefault().isRegistered(this)) {
            EventBus.getDefault().unregister(this);
        }
        if (mTesseractInstance != null) {
            mTesseractInstance.destroy();
        }
        if (mLoginService != null && mLoginBroadcast != null) {
            mLoginService.unregisterLoginReceiver(mLoginBroadcast);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        IUserOperationApi userMonitor = getUserMonitor();
        if (userMonitor != null) {
            userMonitor.pageOut(pageName(), null);
        }
    }

    protected void initRecyclerView() {
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.getContext()) {
            @Override
            public boolean canScrollVertically() {
                return mEnableScrollVertical;
            }
        };
        final RecyclerView recyclerView = mTesseractInstance.getRecyclerView();
        recyclerView.setLayoutManager(layoutManager);

        //获取内容区高度
        ViewTreeObserver observer = recyclerView.getViewTreeObserver();
        observer.addOnPreDrawListener(new ViewTreeObserver.OnPreDrawListener() {
            @Override
            public boolean onPreDraw() {
                recyclerView.getViewTreeObserver().removeOnPreDrawListener(this);
                RVHeightUtils.putRVHeight(pageName(), recyclerView.getHeight());
                return true;
            }
        });
    }

    private void setTitle(String title) {
        if (TextUtils.isEmpty(title)) {
            return;
        }
        if (mNavBar != null) {
            mNavBar.setTitle(title);
        }
    }

    private void showShareBtn(String title) {
        Activity activity = getActivity();
        if (activity == null) {
            return;
        }
        Bundle bundle = activity.getIntent().getExtras();
        if (mNavBar != null && bundle != null && "1".equals(bundle.getString("share"))) {
            mNavBar.showShareBtn(TextUtils.isEmpty(title) ? "" : title, "", bundle.getString("url", ""));
        }
    }


    @Override
    public void onRefresh(RefreshLayout refreshLayout) {
        if (refreshListener != null && this.refreshListener.onRefresh()) {
            return;
        }
        doRefresh(mSinglePageRequestUrl, REQUEST_TYPE_PULL_FRESH);
        mRefreshLayout.finishRefresh(1000);
    }

    @Override
    public void requestData() {
        if (!mEnableResumeRefresh) {
            doRefresh(mSinglePageRequestUrl, REQUEST_TYPE_LOAD);
        }
    }

    public void refresh() {
        doRefresh(mSinglePageRequestUrl, REQUEST_TYPE_NORMAL_REFRESH);
    }

    public void doRefresh(String requestUrl, final String reqType) {
        if (refreshListener != null) {
            refreshListener.onRefresh();
        }
        if (TextUtils.isEmpty(requestUrl)) {
            return;
        }
        if (!mHideLoading) {
            mTesseractInstance.showLoading("");
        }
        if (mMgopRequest) {
            doMgopRequest(requestUrl, reqType);
        } else {
            doNormalRequest(requestUrl, reqType);
        }
    }


    private void doNormalRequest(String requestUrl, final String requestType) {
        final long start = System.currentTimeMillis();
        mTesseractInstance.renderRemoteData(requestUrl, makeHeaders(), new IRequestCallback() {
            @Override
            public void onError(String message, Map<String, ?> map) {
                dealError(message, start);
            }

            @Override
            public void onSuccess(JSONObject jsonObject, Map<String, ?> map) {
                dealSuccess(jsonObject, start, requestType);
            }
        });
    }

    private void doMgopRequest(String requestUrl, final String requestType) {
        final long start = System.currentTimeMillis();
        mTesseractInstance.renderRemoteData(getRequestParamters(requestUrl), makeHeaders(), new IRequestCallback() {
            @Override
            public void onError(String message, Map<String, ?> map) {
                dealError(message, start);
            }

            @Override
            public void onSuccess(JSONObject jsonObject, Map<String, ?> map) {
                dealSuccess(jsonObject, start, requestType);
            }
        });
    }


    private void dealSuccess(JSONObject jsonObject, long startTime, String requestType) {
        long diff = System.currentTimeMillis() - startTime;
        if (!mHideLoading) {
            mTesseractInstance.hideLoading();
        }
        if (!mHideHeaderBar) {
            String title = parseTitleFromGlobal(jsonObject);
            setTitle(title);
            showShareBtn(title);
        }
        toggleRefreshLayoutLoadMore(mTesseractInstance.canLoadMore());
        toggleRefreshLayoutMoreData(!mTesseractInstance.canLoadMore());

        if (jsonObject == null) {
            return;
        }
        if (jsonObject.getBooleanValue("success")) {
            if (mPageCache != null) {
                mPageCache.saveCache(getContext(), pageName(), jsonObject);
            }
        } else if ("3".equals(jsonObject.getString("resultCode"))) {
            if (mLifecycle != null) {
                mLifecycle.onLoginInvaliad();
            }
        }
        //请求打点
        HashMap<String, String> args = new HashMap<>(mMonitorInfo);
        args.put("interval", diff + "");
        args.put("isLoadSuccess", "Y");
        IAppMonitorApi appMonitorApi = getAppMonitor();
        if (appMonitorApi != null) {
            appMonitorApi.commitSuccess(pageName(), "Page_Load_Complete", args);
        }
    }

    private void dealError(String message, long startTime) {
        long diff = System.currentTimeMillis() - startTime;
        if (!mHideLoading) {
            mTesseractInstance.hideLoading();
        }
        ToastUtil.showToast("服务器或网络异常，请稍后重试");
        //请求打点
        HashMap<String, String> args = buildErrorParams(mMonitorInfo, "NETWORK_LOAD_FAIL", message);
        IAppMonitorApi appMonitorApi = getAppMonitor();
        if (appMonitorApi != null) {
            appMonitorApi.commitFail(pageName(), "Page_Load_Error", "Page_Load_Error", message, args);
            args.put("interval", diff + "");
            args.put("isLoadSuccess", "N");
            appMonitorApi.commitSuccess(pageName(), "Page_Load_Complete", args);
        }
    }

    private String parseTitleFromGlobal(JSONObject jsonObject) {
        String title = "";
        if (jsonObject == null) {
            return title;
        }
        JSONObject data = jsonObject.getJSONObject("data");
        if (data == null) {
            return title;
        }
        mLastGlobal = data.getJSONObject("global");
        if (mLastGlobal == null) {
            return title;
        }
        return mLastGlobal.getString("title");
    }

    @Subscribe
    public void modifyFieldsData(ModifyDataEvent event) {
        if (mTesseractInstance == null || !TextUtils.equals(pageName(), event.getTag())) {
            return;
        }
        String fieldId = event.getFieldId();
        if (TextUtils.isEmpty(fieldId)) {
            return;
        }
        IDMComponent component = mTesseractInstance.getDataContext().getComponentByName(fieldId);
        if (component == null) {
            return;
        }
        JSONObject componentFields = component.getFields();
        if (componentFields == null) {
            return;
        }
        JSONObject value = event.getValue();
        if (value == null) {
            return;
        }
        componentFields.putAll(value);
        if (event.isRefreshAfterModity()) {
            mTesseractInstance.refresh(2);
        }
    }

    @Subscribe
    public void dealRefreshEvent(RefreshPageEvent event) {
        if (mTesseractInstance == null || !TextUtils.equals(pageName(), event.tag)) {
            return;
        }
        if (REQUEST_TYPE_SITE_CHANGE.equals(event.refreshType)) {
            refreshBySiteSwitch(event.params);
        } else if (REQUEST_TYPE_NORMAL_REFRESH.equals(event.refreshType)) {
            String request = mSinglePageRequestUrl;
            if (event.params != null && !TextUtils.isEmpty(event.params.getString("url"))) {
                request = event.params.getString("url");
            }
            //刷新当前页面，pageId不能变化
            String pageId = mPageId;
            parseParamsFromUrl(request);
            mPageId = pageId;
            if (request.contains("?")) {
                request += "&siteId=" + mSiteId;
            } else {
                request += "?siteId=" + mSiteId;
            }
            doRefresh(request, REQUEST_TYPE_NORMAL_REFRESH);
        }
    }

    private void refreshBySiteSwitch(JSONObject params) {
        if (params == null) {
            return;
        }

        String siteId = params.getString("siteId");
        if (TextUtils.isEmpty(siteId)) {
            return;
        }
        mSiteId = siteId;
        mSinglePageRequestUrl = replaceUriParameter(mSinglePageRequestUrl, "siteId", siteId);
        doRefresh(mSinglePageRequestUrl, REQUEST_TYPE_SITE_CHANGE);
    }


    private String replaceUriParameter(String url, String key, String newValue) {
        if (TextUtils.isEmpty(url)) {
            return url;
        }
        Uri uri = Uri.parse(url);
        final Set<String> params = uri.getQueryParameterNames();
        final Uri.Builder newUri = uri.buildUpon().clearQuery();
        for (String param : params) {
            newUri.appendQueryParameter(param,
                    param.equals(key) ? newValue : uri.getQueryParameter(param));
        }

        return newUri.build().toString();
    }

    public void refresh(JSONObject renderData) {
        if (renderData != null) {
            mTesseractInstance.renderData(renderData);
        }

    }

    @Override
    public void onLoadMore(RefreshLayout refreshLayout) {
        if (loadMoreListener != null && loadMoreListener.onLoadMore()) {
            return;
        }
        loadMore();
    }

    private void loadMore() {
        if (mEnableLoadMore && mTesseractInstance != null && mTesseractInstance.canLoadMore()) {
            RequestBuilder requestBuilder = new RequestBuilder(mMultiPageRequestUrl);
            requestBuilder.setHeaders(makeHeaders());
            if (mLastGlobal != null) {
                requestBuilder.setBody(new ZWJSONObjectRequestBody(getLoadmoreParams(mSinglePageRequestUrl, mLastGlobal)));
            }
            new Request().execute(requestBuilder.requestPost(), new IRequestCallback() {
                @Override
                public void onError(String s, Map<String, ?> map) {
                    afterLoadMore(mTesseractInstance.canLoadMore());
                }

                @Override
                public void onSuccess(JSONObject jsonObject, Map<String, ?> map) {
                    mTesseractInstance.loadMore(jsonObject);
                    afterLoadMore(mTesseractInstance.canLoadMore());
                }
            });
        } else {
            afterLoadMore(false);
        }
    }

    private String getLoadmoreParams(String rawUrl, JSONObject globalParams) {
        if (TextUtils.isEmpty(rawUrl) && (globalParams == null || globalParams.isEmpty())) {
            return "";
        }
        JSONObject paramsObj = (JSONObject) JSON.toJSON(getRequestParamters(rawUrl));
        if (paramsObj == null) {
            paramsObj = new JSONObject();
        }
        if (globalParams != null && !globalParams.isEmpty()) {
            paramsObj.put("global", globalParams);
        }
        return paramsObj.toString();
    }

    public void loadMore(JSONObject renderData) {
        if (mEnableLoadMore && mTesseractInstance != null && mTesseractInstance.canLoadMore()) {
            mTesseractInstance.loadMore(renderData);
            afterLoadMore(mTesseractInstance.canLoadMore());
        } else {
            afterLoadMore(false);
        }
    }

    public void setEnableLoadMore(boolean enableLoadMore) {
        mEnableLoadMore = enableLoadMore;
        toggleRefreshLayoutLoadMore(enableLoadMore);
    }

    private void toggleRefreshLayoutPullRefresh(boolean flag) {
        if (mRefreshLayout == null) {
            return;
        }
        if (flag) {
            mRefreshLayout.setOnRefreshListener(this);
            mRefreshLayout.setEnableRefresh(true);
        } else {
            mRefreshLayout.setEnableRefresh(false);
        }
    }

    private void toggleRefreshLayoutLoadMore(boolean canLoadMore) {
        if (mRefreshLayout == null) {
            return;
        }
        if (canLoadMore) {
            mRefreshLayout.setOnLoadMoreListener(this);
            mRefreshLayout.setEnableLoadMore(true);
        } else {
            mRefreshLayout.setEnableLoadMore(false);
        }
    }

    private void toggleRefreshLayoutMoreData(boolean noMoreData) {
        if (mRefreshLayout != null) {
            mRefreshLayout.setNoMoreData(noMoreData);
        }
    }

    public void mgopRequestNeedLogin(boolean flag) {
        this.mMgopRequestNeedLogin = flag;
    }

    public void hideHeaderView(boolean flag) {
        this.mHideHeaderBar = flag;
        if (mHeaderLayout != null) {
            mHeaderLayout.setVisibility(flag ? View.GONE : View.VISIBLE);
        }
    }

    public void setEnablePullRefresh(boolean flag) {
        mEnablePullRefresh = flag;
        toggleRefreshLayoutPullRefresh(flag);
    }

    private void afterLoadMore(boolean canLoadMore) {
        if (mRefreshLayout != null) {
            mRefreshLayout.finishLoadMore();
            toggleRefreshLayoutMoreData(!canLoadMore);
        }
    }

    public void setRefreshListener(IRefreshListener refreshListener) {
        this.refreshListener = refreshListener;
    }


    public void setLoadMoreListener(ILoadMoreListener loadMoreListener) {
        this.loadMoreListener = loadMoreListener;
    }

    public void setEventSubscriberMap(Map<String, ISubscriber> mEventSubscriber) {
        this.mEventSubscriber = mEventSubscriber;
    }

    public void setPageName(String pageName) {
        this.mPageName = pageName;
    }

    public void setRequestUrl(String requestUrl) {
        this.mSinglePageRequestUrl = parseAndSetSiteId(requestUrl);
        parseParamsFromUrl(mSinglePageRequestUrl);
    }

    public void enableCache() {
        this.mPageCache = new PageCache();
    }

    public class LoginStateChangeReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            switch (action) {
                case ILoginService.GOV_LOGIN_BROADCAST_ACTION:
                    if (mEnableLoginReceive) {
                        doRefresh(mSinglePageRequestUrl, REQUEST_TYPE_LOGIN);
                    }
                    break;
                case ILoginService.GOV_LOGOUT_BROADCAST_ACTION:
                    if (mPageCache != null) {
                        mPageCache.cleanCache(getContext(), pageName());
                    }
                    break;
            }
        }

    }

    private HashMap<String, String> buildErrorParams(@NonNull Map<String, String> commonParams, @NonNull String errorCode, String errorMessage) {
        HashMap<String, String> map = new HashMap<>(commonParams);
        map.put("ErrorCode", errorCode);
        if (!TextUtils.isEmpty(errorMessage)) {
            map.put("ErrorMessage", errorMessage);
        }
        return map;
    }

    private IUserOperationApi getUserMonitor() {
        IZWMonitor monitor = ServiceManager.getInstance().getService(IZWMonitor.class.getName());
        if (monitor == null) {
            return null;
        }
        return monitor.getUserOperationApi();
    }

    private IAppMonitorApi getAppMonitor() {
        IZWMonitor monitor = ServiceManager.getInstance().getService(IZWMonitor.class.getName());
        if (monitor == null) {
            return null;
        }
        return monitor.getAppMonitorApi();
    }

//    private boolean canDealTokenInvaliad() {
//        return SharedPreferencesUtil.getBoolean("tesseract_deal_token_invaliad", false);
//    }

//    private void toggleDealTokenInvaliad(boolean flag) {
//        SharedPreferencesUtil.putBoolean("tesseract_deal_token_invaliad", flag);
//    }

    private Map<String, String> getRequestParamters(String rawUrl) {
        Map<String, String> map = new HashMap<>();
        Uri rawUri = Uri.parse(rawUrl);
        if (rawUri == null) {
            return map;
        }
        Set<String> paramKeys = rawUri.getQueryParameterNames();
        if (paramKeys == null || paramKeys.isEmpty()) {
            return map;
        }
        Iterator<String> iterator = paramKeys.iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            if (TextUtils.equals(key, "")) {
                continue;
            }
            map.put(key, rawUri.getQueryParameter(key));
        }
        return map;
    }


}
