package com.alibaba.tesseract.page.activity;

import android.content.Intent;
import android.net.Uri;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.text.TextUtils;

import com.alibaba.gov.android.api.zwmonitor.IZWMonitor;
import com.alibaba.gov.android.common.config.AppConfig;
import com.alibaba.gov.android.foundation.base.activity.BaseActivity;
import com.alibaba.gov.android.foundation.utils.SharedPreferencesUtil;
import com.alibaba.gov.android.tesseract.page.R;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import static com.alibaba.tesseract.page.activity.Constants.MULTI_PAGE_REQUEST_API;
import static com.alibaba.tesseract.page.activity.Constants.SINGLE_PAGE_REQUEST_API;

/**
 * Created by yiwei on 2020-04-09.
 */
public class TesseractActivity extends BaseActivity {
    private BaseTesseractFragment mFragment;
    private String mRequestUrl;
    private String mReplaceRequestUrl;
    String U_LATITUDE_FROM_GAODE = "latitude_from_gaode";      //高德定位的坐标
    String U_LONGITUDE_FROM_GAODE = "longitude_from_gaode";    //高德定位的坐标

    @Override
    public void initViews() {
        hideTitleBar();
        instanceFragment();
    }

    @Override
    protected int getLayoutResId() {
        return R.layout.common_tesseract_page_layout;
    }

    @Override
    protected void findViews() {
    }

    @Override
    protected void initData(Intent intent) {
        if (intent == null) {
            return;
        }
        Uri rawUrl = intent.getData();
        if (rawUrl == null) {
            return;
        }
        mReplaceRequestUrl = rawUrl.getQueryParameter("_tesseract_url_");
        mRequestUrl = AppConfig.getString("baseHost") + SINGLE_PAGE_REQUEST_API;

        mRequestUrl = copyParameterToTargetUrl(rawUrl.toString(), TextUtils.isEmpty(mReplaceRequestUrl) ? mRequestUrl : mReplaceRequestUrl);
        //todo 经纬度,需要@寒潇提供经纬度获取接口
        String longitude = SharedPreferencesUtil.getString(U_LONGITUDE_FROM_GAODE, "120.040458");
        String latitude = SharedPreferencesUtil.getString(U_LATITUDE_FROM_GAODE, "21.95");
        mRequestUrl += "&longitude=" + longitude + "&latitude=" + latitude;
    }

    protected void instanceFragment() {
        FragmentManager manager = this.getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        if (mFragment == null) {
            mFragment = new BaseTesseractFragment();
            //api切换mgop开关
//            if (TextUtils.isEmpty(mReplaceRequestUrl)) {
//                mRequestUrl += "&" + MGOP_REQUEST + "=true";
//            }
            mFragment.setRequestUrl(mRequestUrl);
        }
        transaction.add(R.id.common_tesseract_body, mFragment);
        transaction.commitAllowingStateLoss();
    }

    private String copyParameterToTargetUrl(String rawUrl, String targetUrl) {
        List<String> keys = new ArrayList<>();
        keys.add("_tesseract_");
        keys.add("_tesseract_url_");
        return copyParameters(rawUrl, targetUrl, keys);
    }

    private String copyParameters(String rawUrl, String targetUrl, List<String> filterKey) {
        if (TextUtils.isEmpty(rawUrl) || TextUtils.isEmpty(targetUrl)) {
            return "";
        }
        Uri rawUri = Uri.parse(rawUrl);
        Uri targetUri = Uri.parse(targetUrl);
        Set<String> paramKeys = rawUri.getQueryParameterNames();
        if (paramKeys == null || paramKeys.isEmpty()) {
            return null;
        }
        if (filterKey == null) {
            filterKey = new ArrayList<>();
        }
        Iterator<String> iterator = paramKeys.iterator();
        Uri.Builder targetBuilder = targetUri.buildUpon();
        while (iterator.hasNext()) {
            String key = iterator.next();
            if (filterKey.contains(key)) {
                continue;
            }
            if (TextUtils.equals(key, "")) {
                continue;
            }
            targetBuilder.appendQueryParameter(key, rawUri.getQueryParameter(key));
        }
        return targetBuilder.build().toString();
    }
}
