package com.alibaba.tesseract.page.events;


import com.alibaba.android.tesseract.container.vfw.core.ViewEngine;
import com.alibaba.android.tesseract.container.vfw.event.OnDynamicEventListener;
import com.alibaba.android.tesseract.core.event.base.TesseractEvent;
import com.alibaba.android.tesseract.core.event.base.TesseractEventHandler;
import com.alibaba.android.tesseract.sdk.datamodel.imp.DMEvent;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.gov.android.api.router.IRouterService;
import com.alibaba.gov.android.servicebus.manager.ServiceManager;
import com.taobao.android.dinamicx.DXRuntimeContext;
import com.taobao.android.dinamicx.IDXEventHandler;
import com.taobao.android.dinamicx.expression.event.DXEvent;
import com.taobao.android.dinamicx.template.utils.DXHashUtil;

import java.util.Map;

import static com.alibaba.android.tesseract.core.event.base.BaseEventType.EVENT_TYPE_USER_TRACK;


/**
 * Created by yiwei on 2019-12-17.
 */
public class ZWOpenUrlEventHandler implements IDXEventHandler {
    public static final long TAG_ID = DXHashUtil.hash("dxZWOpenUrl");

    @Override
    public void handleEvent(DXEvent event, Object[] args, DXRuntimeContext runtimeContext) {
        if (args == null
                || args.length == 0
                || runtimeContext == null
                || runtimeContext.getContext() == null) {
            return;
        }
        postTrackEvent(runtimeContext, args);
        IRouterService routerService = ServiceManager.getInstance().getService(IRouterService.class.getName());
        if (null != routerService) {
            routerService.goToUri(runtimeContext.getContext(), args[0].toString());
        }
    }


    private void postTrackEvent(DXRuntimeContext runtimeContext, Object[] args) {
        Object dinamicContext = runtimeContext.getDxUserContext();
        if (!(dinamicContext instanceof Map)) {
            return;
        }
        ViewEngine engine = (ViewEngine) ((Map) dinamicContext).get(ViewEngine.KEY_VIEW_ENGINE);
        if (engine == null) {
            return;
        }
        OnDynamicEventListener eventHandler = engine.getService(OnDynamicEventListener.class);
        if (eventHandler == null || !(eventHandler instanceof TesseractEventHandler)) {
            return;
        }
        TesseractEvent tesseractEvent = ((TesseractEventHandler) eventHandler).buildTesseractEvent()
                .setEventType(EVENT_TYPE_USER_TRACK);
        DMEvent event = makeUserTrackEvent(runtimeContext, args);
        if (event == null) {
            return;
        }
        tesseractEvent.setEventParams(event);
        ((TesseractEventHandler) eventHandler).dispatchEvent(tesseractEvent);
    }

    private DMEvent makeUserTrackEvent(DXRuntimeContext runtimeContext, Object[] eventArgs) {
        String area = "dxZWOpenUrl", index = "0";
        if (eventArgs.length == 3) {
            if (eventArgs[1] != null) {
                area = eventArgs[1].toString();
            }
            if (eventArgs[2] != null) {
                index = eventArgs[2].toString();
            }
        }
        JSONObject data = runtimeContext.getData();
        if (data == null || data.isEmpty()) {
            return null;
        }
        String arg1 = data.getString("type") + "_" + data.getString("id") + "_" + area;
        JSONObject eventFields = new JSONObject();
        eventFields.put("eventId", "2101");
        eventFields.put("arg1", arg1);
        JSONObject args = new JSONObject();
        if (data != null && data.getJSONObject("fields") != null) {
            args = (JSONObject) data.getJSONObject("fields").clone();
        }
        args.put("index", index);
        eventFields.put("args", args);
        return new DMEvent(EVENT_TYPE_USER_TRACK, eventFields, null);
    }

    @Override
    public void prepareBindEventWithArgs(Object[] args, DXRuntimeContext runtimeContext) {

    }
}
