package com.alibaba.tesseract.page.events.site;

import android.annotation.SuppressLint;
import android.content.Context;
import android.text.TextUtils;

import com.alibaba.android.tesseract.core.utils.LogUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.gov.android.api.network.IZWHttpService;
import com.alibaba.gov.android.api.network.ZWBaseApi;
import com.alibaba.gov.android.api.network.request.ZWRequest;
import com.alibaba.gov.android.api.network.response.ZWResponse;
import com.alibaba.gov.android.api.site.ISiteMidService;
import com.alibaba.gov.android.api.site.SiteAPI;
import com.alibaba.gov.android.api.site.SiteModel;
import com.alibaba.gov.android.api.site.SiteSelectionCallback;
import com.alibaba.gov.android.foundation.utils.ToastUtil;
import com.alibaba.gov.android.servicebus.manager.ServiceManager;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.reactivex.Observable;
import io.reactivex.ObservableSource;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.annotations.NonNull;
import io.reactivex.functions.Consumer;
import io.reactivex.functions.Function;
import io.reactivex.schedulers.Schedulers;

/**
 * Created by yiwei on 2021/4/1.
 */
public class SiteSwitchHelper {
    private Context mContext;
    private ISiteMidService siteMidService;
    private String selectCode;
    private int flag;
    private String siteUrl;
    private String siteParentUrl;
    private static int startLevel;
    private static int endLevel;

    public SiteSwitchHelper(Context context, String selectCode, int flag, String siteUrl, String siteParentUrl, int startLevel, int endLevel) {
        this.mContext = context;
        this.siteMidService = ServiceManager.getInstance().getService(ISiteMidService.class.getName());
        if (TextUtils.isEmpty(selectCode)) {
            this.selectCode = siteMidService.getGlobalSite().adCode;
        } else {
            this.selectCode = selectCode;
        }
        this.flag = flag;
        this.siteUrl = siteUrl;
        this.siteParentUrl = siteParentUrl;

        if (endLevel <= 0) {
            this.endLevel = 3;
        } else {
            this.endLevel = endLevel;
        }

    }

    public void startSiteMidBySearchResult(SiteSelectionCallback callback) {
        Map traceArgs = new HashMap();
        traceArgs.put("from", "Page_Tesseract");
        this.siteMidService.show(this.mContext, selectCode, new SearchAllSiteAPIImpl(), this.flag, traceArgs, callback);
    }


    class SearchAllSiteAPIImpl implements SiteAPI {
        @SuppressLint({"CheckResult"})
        public void fetchSiteData(String code, final SiteAPICallback callback) {
            queryChildDistrictByidtAPI(code, siteUrl).subscribe(new Consumer<JSON>() {
                @Override
                public void accept(JSON result) throws Exception {
                    if (result != null && result instanceof JSONArray) {
                        List<SiteModel> list = JSON.parseArray(result.toJSONString(), SiteModel.class);
                        for (SiteModel siteModel : list) {
                            if (siteModel.level >= SiteSwitchHelper.endLevel) {
                                siteModel.isLeaf = true;
                            }
                        }
                        callback.callback(list, null);
                    } else {
                        callback.callback((List) null, "获取站点失败");
                    }
                }
            }, new Consumer<Throwable>() {
                @Override
                public void accept(Throwable throwable) throws Exception {
                    LogUtils.e(throwable.getLocalizedMessage());
                }
            });
        }

        @SuppressLint("CheckResult")
        public void fetchParentData(String code, final SiteAPICallback callback) {
            queryDistrictPathById(code, siteParentUrl).subscribe(new Consumer<JSON>() {
                @Override
                public void accept(JSON result) throws Exception {
                    if (result != null && result instanceof JSONArray) {
                        List<SiteModel> list = JSON.parseArray(result.toJSONString(), SiteModel.class);
                        int size = list.size();
                        for (int i = size - 1; i >= 0; i--) {
                            SiteModel siteModel = list.get(i);
                            if (siteModel.level >= SiteSwitchHelper.endLevel) {
                                list.remove(siteModel);
                            } else {
                                callback.callback(list, null);
                                return;
                            }

                        }
                        callback.callback(list, null);
                    } else {
                        callback.callback(null, "获取站点失败");
                    }
                }
            }, new Consumer<Throwable>() {
                @Override
                public void accept(Throwable throwable) throws Exception {
                    LogUtils.e(throwable.getLocalizedMessage());
                }
            });
        }

        public Observable<JSON> queryChildDistrictByidtAPI(String adCode, String siteUrl) {
            SiteMidQueryChildDistrictByidtAPI api = new SiteMidQueryChildDistrictByidtAPI(adCode, siteUrl);
            return this.request(api);
        }


        public Observable<JSON> queryDistrictPathById(String code, String siteParentUrl) {
            SearchQueryDistrictPathByIdAPI api = new SearchQueryDistrictPathByIdAPI(code, siteParentUrl);
            return request(api);
        }


        @SuppressLint({"CheckResult"})
        public Observable<JSON> request(ZWBaseApi api) {
            IZWHttpService zwHttpService = ServiceManager.getInstance().getService(IZWHttpService.class.getName());
            return zwHttpService.execute(api).flatMap(new Function<ZWResponse<?>, ObservableSource<JSON>>() {
                @Override
                public ObservableSource<JSON> apply(@NonNull ZWResponse<?> response) throws Exception {
                    JSON data = null;
                    try {
                        if (response != null && response.getResult() != null) {
                            JSONObject responseJO = JSONObject.parseObject((String)response.getResult());
                            if (responseJO != null && responseJO.getBoolean("success")) {
                                Object dataObj = responseJO.get("data");
                                if (dataObj == null) {
                                    throw new Exception("获取数据失败！");
                                }
                                if( dataObj instanceof JSON) {
                                    data = (JSON) dataObj;
                                }
                            } else {
                                ToastUtil.showToast(responseJO.getString("errorMsg"));
                            }
                        }
                    } catch (Exception var4) {
                        LogUtils.e(var4.getMessage());
                        var4.printStackTrace();
                    }
                    return Observable.just(data != null ? data : new JSONObject());
                }
            }).subscribeOn(Schedulers.newThread()).observeOn(AndroidSchedulers.mainThread());
        }
    }

    class SiteMidQueryChildDistrictByidtAPI extends ZWBaseApi {
        private String apiUrl;
        private String parentCode;

        public SiteMidQueryChildDistrictByidtAPI(String adCode, String siteUrl) {
            this.parentCode = adCode;
            this.apiUrl = siteUrl;
        }

        public ZWRequest request() {
            String requestUrl = apiUrl + this.parentCode;
            LogUtils.d("zwfw.SiteMidQueryAllChildSiteOnlineTransactAPI -> " + requestUrl);
            return (new ZWRequest.Builder(requestUrl)).get().build();
        }
    }

    class SearchQueryDistrictPathByIdAPI extends ZWBaseApi {

        private final String TAG = "zwfw.SearchQueryDistrictPathByIdAPI";

        private String siteParentUrl;
        private String code;

        public SearchQueryDistrictPathByIdAPI(String code, String siteParentUrl) {
            this.code = code;
            this.siteParentUrl = siteParentUrl;
        }

        @Override
        public ZWRequest request() {
            String requestUrl = siteParentUrl + code;
            LogUtils.d(TAG + " -> " + requestUrl);
            return (new ZWRequest.Builder(requestUrl)).get().build();
        }
    }
}
